'''
Argbloks provides declarative style building block for commandline parsing.
Idea is to have common set of arguments and options, with consistent wording
thru various commands.
'''
from os import path


BASEDIR = path.realpath(path.dirname(__file__))
VERSION_FILENAME = path.join(BASEDIR, 'VERSION')


def read_version():
    'To get version out of robocode/VERSION file'
    with open(VERSION_FILENAME) as source:
        return source.read().strip()


OPTIONAL_VARIABLES_ARGUMENT = '''
json file for environment variables (optional)
'''


def optional_variables_argument(parser):
    'builder function for optional --variables argument'
    parser.add_argument('--variables', '-v', type=str,
                        help=OPTIONAL_VARIABLES_ARGUMENT)
    return parser


def optional_force_flag(parser):
    'builder function for optional --force flag'
    parser.add_argument(
        '--force', action='store_true',
        help='''
        remove all safety nets and trust that you know what you are doing
        '''.strip())
    return parser


def optional_version_flag(parser):
    'builder function for optional --version flag'
    version = '%%(prog)s v%s' % read_version()
    parser.add_argument('--version', '-v', action='version', version=version)
    return parser


def optional_debug_flag(parser):
    'builder function for optional --debug flag'
    parser.add_argument(
        '--debug', action='store_true',
        help='be more verbose on output (for debugging purposes)')
    return parser


def optional_silent_flag(parser):
    'builder function for optional --silent flag'
    parser.add_argument(
        '--silent', action='store_true',
        help='do action silently, without showing any output')
    return parser


def optional_json_flag(parser):
    'builder function for optional --json flag'
    parser.add_argument('--json', '-j', action='store_true',
                        help='output result in json format')
    return parser


def optional_dryrun_flag(parser):
    'builder function for optional --dryrun flag'
    parser.add_argument(
        '--dryrun', action='store_true',
        help='do not actually do actions, just show what would be done')
    return parser


def default_config_argument(parser):
    'builder function for --config argument with default value'
    parser.add_argument(
        '--config', '-c', type=str, default='~/.robocode.yaml',
        help='path to configuration file')
    return parser


def default_account_argument(parser):
    'builder function for --account argument with default value'
    parser.add_argument('--account', '-a', type=str, default='default',
                        help='Account to use with this command.')
    return parser


def default_minutes_argument(parser):
    'builder function for --minutes argument with default value'
    parser.add_argument('--minutes', '-m', type=int, default=5,
                        help='how many minutes authorization should be valid')
    return parser


DEFAULT_ENDPOINT = 'https://api.eu1.robocloud.eu/'
ENDPOINT_HELP = '''
Robocloud endpoint that this credentials belongs to. Default value should work
for most users without any change.

SO, DO NOT CHANGE THIS, UNLESS YOU REALLY KNOW WHAT YOU ARE DOING!
'''


def default_endpoint_argument(parser):
    'builder function for --endpoint argument with default value'
    parser.add_argument('--endpoint', '-e', type=str,
                        default=DEFAULT_ENDPOINT,
                        help=ENDPOINT_HELP.strip())
    return parser


def default_workarea_argument(parser):
    'builder function for --workarea argument with default value'
    description = 'path to directory to be used as workarea (default: temp)'
    parser.add_argument('--workarea', type=str, default='temp',
                        help=description)
    return parser


def default_ignore_argument(parser):
    'builder function for --ignore argument with default value'
    parser.add_argument('--ignore', '-i', type=str, default=None,
                        help='filename for ignore patterns')
    return parser


def default_package_argument(parser):
    'builder function for --package argument with default value'
    description = 'filename for target package (default: package.zip)'
    parser.add_argument('--package', '-p', type=str, default='package.zip',
                        help=description)
    return parser


def default_directory_argument(parser):
    'builder function for --directory argument with default value'
    parser.add_argument('--directory', '-d', type=str, default='.',
                        help='directory for command to use (default: .)')
    return parser


def mandatory_positional_entrypoint_argument(parser):
    'builder function for mandatory positional entrypoint argument'
    parser.add_argument('entrypoint', type=str, nargs=1,
                        help='entrypoint to execute in this run')
    return parser


def mandatory_workspace_argument(parser):
    'builder function for mandatory --workspace argument'
    parser.add_argument('--workspace', '-w', type=int, required=True,
                        help='integer workspace id to use with this command')
    return parser


def mandatory_activity_argument(parser):
    'builder function for mandatory --activity argument'
    parser.add_argument('--activity', type=int, required=True,
                        help='integer activity id to use with this command')
    return parser


def optional_workspace_argument(parser):
    'builder function for optional --workspace argument'
    parser.add_argument('--workspace', type=int, default=None,
                        help='integer workspace id for authorization')
    return parser


def optional_positional_credentials_argument(parser):
    'builder function for optional credentials argument'
    parser.add_argument('credentials', type=str, default=None, nargs='?',
                        help='Robocloud API credentials to store.')
    return parser


def optional_positional_directory_argument(parser):
    'builder function for optional directory argument'
    parser.add_argument('directory', type=str, nargs='?', default=None,
                        help='directory to used for this command')
    return parser


# libs related arguments follows:


DEFAULT_CONDA_ARGUMENT_HELP = '''
optional path to environment yaml file (default: config/conda.yaml)
'''.strip()


def default_conda_argument(parser):
    'builder function for --conda argument with default value'
    parser.add_argument('--conda', type=str,
                        default='config/conda.yaml',
                        help=DEFAULT_CONDA_ARGUMENT_HELP)
    return parser


def default_positional_detail_argument(parser):
    'builder function for positional detail argument with default value'
    parser.add_argument('detail', type=str, nargs='?', default=None,
                        help='detail to add or remove')
    return parser


def optional_name_argument(parser):
    'builder function for --name argument with default value'
    parser.add_argument('--name', '-n', type=str, default=None,
                        help='change name of configuration')
    return parser


def optional_add_flag(parser):
    'builder function for --add argument with default value'
    parser.add_argument('--add', action='store_true',
                        help='add new library as requirement')
    return parser


def optional_remove_flag(parser):
    'builder function for --remove argument with default value'
    parser.add_argument('--remove', '-r', action='store_true',
                        help='remove library from requirements')
    return parser


OPTIONAL_PIP_FLAG_HELP = '''
operate on pip packages (default is conda packages)
'''.strip()


def optional_pip_flag(parser):
    'builder function for --pip argument with default value'
    parser.add_argument('--pip', '-p', action='store_true',
                        help=OPTIONAL_PIP_FLAG_HELP)
    return parser


def optional_channel_flag(parser):
    'builder function for --channel argument with default value'
    parser.add_argument('--channel', action='store_true',
                        help='operate on channels (default is packages)')
    return parser
