'''
Module provides authorization decorator that can wrap commands and
provide bearer token authorization to those commands that have
authorization_capability variable correctly set to required capabilites.
'''
from base64 import b64encode
from hashlib import sha256
import hmac
import json
import pathlib
import time
from .commands.abstract_ import CommandBase
from . import configure


UTF_8 = 'utf-8'
CONTENT_TYPE = 'Content-Type'
APPLICATION_JSON = 'application/json'
AUTHORIZATION_HEADER = 'Authorization'
NONCE_HEADER = 'authorization-timestamp'
API_PATH = '/token-vendor-v1/workspaces/%(workspace)s/tokenrequest'
API_FORMAT = '%(endpoint)s' + API_PATH
API_METHOD = 'POST'
NEWLINE = '\n'


def robocloud_hmac(identifier, token):
    "Wraps non-empty token with robocloud hmac prefix"
    if identifier and token:
        return 'robocloud-hmac %s %s' % (identifier, token)
    return None


def bearer(token):
    "Wraps non-empty token with Bearer prefix"
    return 'Bearer %s' % token if token else None


def digest(content):
    'create base 64 encoded SHA 256 digest from content'
    return b64encode(sha256(content).digest()).decode(UTF_8)


def header(headers, name):
    'Extract named header from given set of headers'
    return headers.get(name, None) or headers.get(name.lower(), None)


class AuthorizationDecorator(CommandBase):
    '''
    Decorator class to provide authorize header needed inside core command.
    '''
    def __init__(self, core):
        super(AuthorizationDecorator, self).__init__(core._context.copy())
        self._core = core

    def __call__(self, context):
        caps = self._core.authorization_capability
        if caps:
            endpoint, token = self._authorize(caps, context)
            context['endpoint'] = endpoint
            context['authorization'] = token
        return self._core(context)

    def _authorize(self, caps, context):
        local_context = self._full_context(context)
        configfile = local_context.get('config', None)
        account_name = local_context.get('account', None)
        workspace = local_context.get('workspace', None)
        self._validate_options(configfile, account_name, workspace)
        config = configure.Config()
        config.load_from(configfile)
        account = config.find_account(account_name)
        self._validate_account(account, account_name)
        url = self._token_vendor_url(account.endpoint, workspace)
        https = self._ensure_https(url)
        _, token, reply = self._as_token(https, account, workspace, caps)
        reply['endpoint'] = account.endpoint
        context['@authorization'] = reply
        return account.endpoint, token

    def _validate_options(self, configfile, account_name, workspace):
        if not account_name:
            raise ValueError('Value for account is missing (--account).')
        if not configfile:
            raise ValueError('Config file name is missing (--config).')
        fullpath = pathlib.Path(configfile).expanduser().absolute()
        if not fullpath.is_file():
            note = 'Cannot see file: %s' % fullpath
            raise FileNotFoundError(note)
        if not workspace:
            raise ValueError('Value for workspace is missing (--workspace).')
        return True

    def _validate_account(self, account, account_name):
        if not account:
            note = 'Account not found: %r' % account_name
            raise ValueError(note)
        if not account.endpoint:
            note = 'No endpoint for account. Maybe misconfiguration.'
            raise ValueError(note)
        if not account.identifier:
            message = 'No identifier for account. Maybe misconfiguration.'
            raise ValueError(message)
        if not account.secret:
            raise ValueError('No secret for account. Maybe misconfiguration.')
        return True

    def _token_vendor_url(self, endpoint, workspace):
        # pylint: disable=unused-argument
        endpoint = endpoint.rstrip('/')
        result = API_FORMAT % locals()
        return result.strip()

    def _token_vendor_path(self, workspace):
        # pylint: disable=unused-argument
        result = API_PATH % locals()
        return result.strip()

    def _as_token(self, https, account, workspace, caps):
        nonce = str(int(time.time()))
        body = json.dumps(caps).encode(UTF_8)
        signed = self._hmac_signature(account.secret, nonce, workspace,
                                      digest(body))
        headers = {AUTHORIZATION_HEADER: robocloud_hmac(account.identifier,
                                                        signed),
                   CONTENT_TYPE: APPLICATION_JSON,
                   NONCE_HEADER: nonce}
        response = self._core.web.post(https, headers=headers, data=body)
        self._validate_response(response)
        content = response.json()
        token = content.get('token', None)
        content['status'] = response.status_code
        content['requested'] = caps
        return response.status_code, bearer(token), content

    def _validate_response(self, response):
        if response.status_code != 200:
            form = 'Authorization failed with status %r:\n%s'
            raise ConnectionError(form % (response.status_code, response.text))
        content_type = header(response.headers, 'Content-Type')
        if content_type != 'application/json':
            form = 'Received bad Content-Type %r:\n%s'
            raise TypeError(form % (content_type, response.text))
        return True

    def _hmac_signature(self, secret, nonce, workspace, body_digest):
        api_path = self._token_vendor_path(workspace)
        parts = (API_METHOD, api_path, APPLICATION_JSON, nonce, body_digest)
        payload = NEWLINE.join(parts)
        digger = hmac.new(secret.encode(UTF_8), payload.encode(UTF_8), sha256)
        return b64encode(digger.digest()).decode(UTF_8)
