'''
This module provides activities command to track users activity directories.
'''

import json
import pathlib
import sys
import time
from collections import namedtuple
from .. import configure
from .abstract_ import CommandBase, everyparser
from .credentials import validate_configfile

Sorter = namedtuple('Sorter', 'timestamp activity')


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by activities command.
    '''
    argbloks.optional_positional_directory_argument(parser)
    argbloks.optional_json_flag(parser)
    return everyparser(parser, argbloks)


def touch(config, directory):
    '''
    To touch activity given directory with given config.
    '''
    activity = Command(dict(config=config, directory=directory))
    return activity(dict())


class Command(CommandBase):
    '''
    Activities command lists or updates tracked activity directories.
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual activity structure.
        '''
        local_context = self._full_context(context)
        configfile = local_context.get('config', None)
        directory = local_context.get('directory', None)
        validate_configfile(configfile)
        if directory:
            return self._touch_activity(directory, configfile)
        return self._show_activities(configfile, local_context)

    def _touch_activity(self, directory, configfile):
        config = configure.Config()
        config.load_from(configfile)
        config.summon_activity(directory)
        config.save_as(configfile)
        return True

    def _show_activities(self, configfile, context):
        config = configure.Config()
        config.load_from(configfile)
        jsonic = context.get('json', False)
        activities = self._reorder_activities(config.activities)
        if self._update_deleted(config, activities):
            config.save_as(configfile)
            activities = self._reorder_activities(config.activities)
        if jsonic:
            json.dump([x.as_dict for x in activities], sys.stdout, indent=2)
        else:
            self._textual_report(activities)
        return True

    def _textual_report(self, activities):
        if activities:
            print('Updated at      ', '|', 'Created at      ',
                  '|', 'Activity directory')
        for activity in activities:
            updated = time.strftime('%d.%m.%Y %H:%M',
                                    time.localtime(activity.updated))
            created = time.strftime('%d.%m.%Y %H:%M',
                                    time.localtime(activity.created))
            deleted = ' <deleted>' if activity.deleted else ''
            print(updated, '|', created, '|', activity.activity + deleted)

    def _update_deleted(self, config, activities):
        updated = False
        for activity in activities:
            fullpath = pathlib.Path(activity.activity)
            if activity.deleted is None and not fullpath.is_dir():
                config.delete_activity(fullpath)
                updated = True
            elif activity.deleted is not None and fullpath.is_dir():
                config.summon_activity(fullpath)
                updated = True
        return updated

    def _reorder_activities(self, activities):
        stamped = list()
        for activity in activities:
            stamped.append(Sorter(activity.updated, activity))
        return tuple(x.activity for x in sorted(stamped))
