'''
This module provides credentials command to store Robocloud tokens into
CLI configuration file.
'''

import json
import pathlib
from robocode import configure
from .abstract_ import CommandBase, everyparser

SHOWTITLE = 'Identifier    Account     Secret              Endpoint'
SHOWFORM = '  %(identifier)-10s  %(account)-10s  %(secret)-18s  %(endpoint)s'


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by credentials command.
    '''
    argbloks.optional_json_flag(parser)
    argbloks.default_endpoint_argument(parser)
    argbloks.optional_positional_credentials_argument(parser)
    return everyparser(parser, argbloks)


def validate_configfile(configfile):
    '''
    Supporting function to validate what config file actually is.
    '''
    if not configfile:
        raise ValueError('Config file name is missing (--config).')
    fullpath = pathlib.Path(configfile).expanduser().absolute()
    if not fullpath.parent.is_dir():
        note = 'Invalid directory for configuration: %s' % fullpath.parent
        raise NotADirectoryError(note)


class Command(CommandBase):
    '''
    Credentials command saves Robocloud API credentials for future use.
    '''
    def __call__(self, context):
        local_context = self._full_context(context)
        configfile = local_context.get('config', None)
        endpoint = local_context.get('endpoint', None)
        account = local_context.get('account', None)
        credentials = local_context.get('credentials', None)
        identifier, secret = self._split_credentials(credentials)
        json_flag = local_context.get('json', None)
        validate_configfile(configfile)
        show = not identifier and not secret
        self._validate(endpoint, account, show, identifier, secret)
        if show:
            return self._show(configfile, json_flag)
        return self._update(configfile, account, endpoint, identifier, secret)

    def _split_credentials(self, credentials):
        if not credentials:
            return None, None
        return credentials.split(':', 1)

    def _show(self, configfile, json_flag):
        if json_flag:
            return self._show_json(configfile)
        return self._show_text(configfile)

    def _account_info(self, account):
        return dict(identifier=account.identifier, account=account.account,
                    secret=account.secret[:15] + '...',
                    endpoint=account.endpoint)

    def _show_text(self, configfile):
        config = configure.Config()
        config.load_from(configfile)
        print(SHOWTITLE)
        for here in sorted(config.accounts):
            print(SHOWFORM % self._account_info(here))
        return True

    def _show_json(self, configfile):
        config = configure.Config()
        config.load_from(configfile)
        entries = list()
        for here in sorted(config.accounts):
            entries.append(self._account_info(here))
        print(json.dumps(entries, indent=2))
        return True

    def _update(self, configfile, account, endpoint, identifier, secret):
        # pylint: disable=too-many-arguments
        config = configure.Config()
        config.load_from(configfile)
        target = configure.Account(account=account, endpoint=endpoint,
                                   identifier=identifier, secret=secret)
        config.update_account(target)
        config.save_as(configfile)
        return True

    def _validate(self, endpoint, account, show, identifier, secret):
        # pylint: disable=too-many-arguments
        if not endpoint:
            raise ValueError('Value for endpoint is missing (--endpoint).')
        self._ensure_https(endpoint)
        self._ensure_api_endpoint(endpoint)
        if not account:
            raise ValueError('Value for account is missing (--account).')
        if show:
            return True
        if not identifier:
            raise ValueError('Value for identifier is missing (identifier).')
        if not secret:
            raise ValueError('Value for secret is missing (secret).')
        return True
