'''
This module provides init command to bootstrap a new rpa package directory
structure.
'''

import shutil
from pathlib import Path
from .abstract_ import CommandBase, everyparser
from .fix import quickfix
from .activities import touch


TEMPLATE_DIR = Path(__file__).parent.parent.joinpath('template').absolute()


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by init command.
    '''
    argbloks.optional_positional_directory_argument(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Init command creates directory structure and CLI support structures.
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual directory init.
        '''
        local_context = self._full_context(context)
        directory = local_context.get('directory', None)
        configfile = local_context.get('config', None)
        self._validate(directory)
        target = Path(directory).absolute()
        target.mkdir(parents=True, exist_ok=True)
        self._copytree(TEMPLATE_DIR.absolute(), target)
        touch(configfile, target)
        return quickfix(target)

    def _validate(self, directory):
        "This method validates that given directory is in correct state."
        if not directory:
            raise NotADirectoryError('Required argument directory is missing!')
        target = Path(directory).absolute()
        if not target.exists():
            return True
        if not target.is_dir():
            raise NotADirectoryError('%r is not a directory!' % directory)
        if len(tuple(target.iterdir())) > 0:
            raise FileExistsError('Directory %r is not empty!' % directory)
        return True

    def _copytree(self, source, target):
        "Recursive function to do actual content copy."
        for entry in source.iterdir():
            if entry.is_dir():
                subdir = target.joinpath(entry.name)
                subdir.mkdir()
                self._copytree(entry, subdir)
            else:
                shutil.copy(entry, target)
