'''
This module provides libs command to handle library management for package.
'''

import json
import pathlib
import sys
import yaml
from .abstract_ import CommandBase, everyparser


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by libs command.
    '''
    argbloks.default_conda_argument(parser)
    argbloks.optional_name_argument(parser)
    argbloks.optional_add_flag(parser)
    argbloks.optional_remove_flag(parser)
    argbloks.optional_pip_flag(parser)
    argbloks.optional_channel_flag(parser)
    argbloks.optional_json_flag(parser)
    argbloks.default_positional_detail_argument(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Libs command manages library dependencies in action oriented way.
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual lib handling wrapping.
        '''
        local_context = self._full_context(context)
        conda = local_context.get('conda')
        add = local_context.get('add', False)
        remove = local_context.get('remove', False)
        pip = local_context.get('pip', False)
        channel = local_context.get('channel', False)
        action = self._select_action(add, remove, pip, channel)
        config = self._load_config(conda)
        renamed = self._rename(config, local_context)
        if action(config, local_context) or renamed:
            self._save_config(conda, config)
        return True

    def _rename(self, config, context):
        name = context.get('name', None)
        visible = str(name).strip()
        if name and visible:
            config['name'] = visible
            return True
        return False

    def _select_action(self, add, remove, pip, channel):
        show = not any((add, remove))
        conda = not any((pip, channel))
        bad = all((add, remove)) or all((pip, channel))
        name = list(filter(bool,
                           [bad and '_bad', show and '_show', add and '_add',
                            remove and '_remove', conda and '_conda',
                            pip and '_pip', channel and '_channel']))
        return getattr(self, ''.join(name))

    def _load_config(self, filename):
        filepath = pathlib.Path(filename)
        if not filepath.is_file():
            return dict()
        with open(filepath) as source:
            return yaml.safe_load(source)

    def _save_config(self, filename, content):
        with open(filename, 'w+') as sink:
            yaml.safe_dump(content, sink, indent=4)
        return True

    def _dump(self, content, context):
        jsonic = context.get('json', False)
        if jsonic:
            json.dump(content, sys.stdout, indent=4)
        else:
            yaml.safe_dump(content, sys.stdout, indent=4)
        return False

    def _show_conda(self, config, context):
        return self._dump(config, context)

    def _show_channel(self, config, context):
        return self._dump(config.get('channels', []), context)

    def _find_pip(self, config, force=False):
        dependencies = config.get('dependencies', [])
        for index, entry in enumerate(dependencies):
            if isinstance(entry, dict) and 'pip' in entry:
                return index, entry.get('pip', [])
        index, result = None, []
        if force:
            config['dependencies'] = dependencies
            index = len(dependencies)
            dependencies.append(dict(pip=result))
        return None, result

    def _show_pip(self, config, context):
        _, found = self._find_pip(config)
        return self._dump(found, context) if found else False

    def _remove_prefixed(self, target, prefix):
        removable = list()
        for index, value in enumerate(target):
            if isinstance(value, str) and value.startswith(prefix):
                removable.insert(0, index)
        for index in removable:
            target.pop(index)
        return len(removable) > 0

    def _remove_conda(self, config, context):
        prefix = context.get('detail', None)
        if not prefix:
            return False
        target = config.get('dependencies', [])
        result = self._remove_prefixed(target, prefix)
        if result and not target:
            config.pop('dependencies')
        return result

    def _remove_channel(self, config, context):
        prefix = context.get('detail', None)
        if not prefix:
            return False
        target = config.get('channels', [])
        result = self._remove_prefixed(target, prefix)
        if result and not target:
            config.pop('channels')
        return result

    def _remove_pip(self, config, context):
        prefix = context.get('detail', None)
        if not prefix:
            return False
        index, target = self._find_pip(config)
        status = self._remove_prefixed(target, prefix)
        if index is not None and not target:
            dependencies = config.get('dependencies', [])
            dependencies.pop(index)
            if not dependencies:
                config.pop('dependencies')
        return status

    def _direct_extract(self, config, name):
        target = config.get(name, [])
        return True, None, target

    def _pip_extract(self, config, _):
        return (False,) + self._find_pip(config, True)

    def _target_add(self, name, config, context, extractor):
        detail = context.get('detail', None)
        if not detail:
            return False
        add, _, target = extractor(config, name)
        if add and not target:
            config[name] = target
        if detail in target:
            return False
        target.append(detail)
        return True

    def _add_conda(self, config, context):
        return self._target_add('dependencies', config, context,
                                self._direct_extract)

    def _add_channel(self, config, context):
        return self._target_add('channels', config, context,
                                self._direct_extract)

    def _add_pip(self, config, context):
        return self._target_add(None, config, context,
                                self._pip_extract)

    # following are all error cases

    def _bad_show_pip_channel(self, _config, _context):
        message = 'Cannot do both --pip and --channel at same time.'
        raise ValueError(message)

    def _bad_remove_pip_channel(self, _config, _context):
        message = 'Cannot do both --pip and --channel at same time.'
        raise ValueError(message)

    def _bad_add_pip_channel(self, _config, _context):
        message = 'Cannot do both --pip and --channel at same time.'
        raise ValueError(message)

    def _bad_add_remove_conda(self, _config, _context):
        message = 'Cannot do both --add and --remove at same time.'
        raise ValueError(message)

    def _bad_add_remove_channel(self, _config, _context):
        message = 'Cannot do both --add and --remove at same time.'
        raise ValueError(message)

    def _bad_add_remove_pip(self, _config, _context):
        message = 'Cannot do both --add and --remove at same time.'
        raise ValueError(message)

    def _bad_add_remove_pip_channel(self, _config, _context):
        message = 'Cannot do both --add and --remove at same time.'
        raise ValueError(message)
