'''
This module provides run command to locally run given package or current
package under construction.
'''

import json
import os
import pathlib
import subprocess
import sys
from .. import main
from .abstract_ import CommandBase, everyparser
from .unwrap import Command as Unwrap
from .authorize import Command as Authorize
from .tooling_ import make_path, cleanup, find_all_entrypoints, stringify
from .tooling_ import is_executable, EXECUTABLES
from .validators_ import validate_optional_variables_argument
from .validators_ import validate_mandatory_workarea_argument


PATH = 'PATH'
PYTHONPATH = 'PYTHONPATH'

PYTHONPATH_EXTENSIONS = ('resources', 'libraries', 'tasks', 'variables')

ENDPOINT_VARIABLES = ('RC_API_SECRET_HOST', 'RC_API_WORKITEM_HOST')
TOKEN_VARIABLES = ('RC_API_SECRET_TOKEN', 'RC_API_WORKITEM_TOKEN')
NOT_AVAILABLE = 'N/A'


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by run command.
    '''
    argbloks.default_package_argument(parser)
    argbloks.optional_variables_argument(parser)
    argbloks.default_workarea_argument(parser)
    argbloks.mandatory_positional_entrypoint_argument(parser)
    argbloks.optional_workspace_argument(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Run command executes specified activity (entrypoint).
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual run structure.
        '''
        local_context = self._full_context(context)
        package = local_context.get('package', None)
        variables = local_context.get('variables', None)
        entrypoint = local_context.get('entrypoint', None)
        workarea = local_context.get('workarea', None)
        validate_optional_variables_argument(variables)
        validate_mandatory_workarea_argument(workarea)
        self._validate_options(package, entrypoint)
        workdir = pathlib.Path(workarea).absolute().joinpath('robocode')
        entrypoints, environment = self._setup(workdir, package, variables,
                                               local_context)
        entrypoints = self._validate_entrypoints(entrypoints)
        fullpath = self._smart_entrypoint_path(entrypoints, entrypoint.pop(0))
        result = self._execute(entrypoints.parent, environment, fullpath)
        return result == 0

    def _validate_entrypoints(self, entrypoints):
        'Entrypoints should always be valid directory, hence validation'
        if len(entrypoints) > 1:
            raise ValueError('Too many entrypoints found inside a package.')
        selected = entrypoints[0] if entrypoints else None
        if not selected:
            raise ValueError('Could not find entrypoints from package.')
        if not selected.is_dir():
            raise NotADirectoryError(
                'Entrypoints is not a directory: %s' % selected)
        return selected

    def _validate_options(self, package, entrypoint):
        '''
        Incoming package, and entrypoint must always
        be given by user, hence this validation.
        '''
        if not package:
            raise ValueError('Missing package name (--package).')
        if not entrypoint:
            raise ValueError('Missing entrypoint name (--entrypoint).')
        if not pathlib.Path(package).is_file():
            form = 'Missing package file: %s. Use "wrap" to create it.'
            raise FileNotFoundError(form % package)
        return True

    def _smart_entrypoint_path(self, directory, entrypoint):
        primary = directory.joinpath(entrypoint)
        if primary.is_file():
            return str(primary)
        secondary = directory.parent.joinpath(entrypoint)
        if secondary.is_file():
            form = 'Warning: you can just use short form %r as entrypoint.\n'
            sys.stderr.write(form % secondary.name)
            return str(secondary)
        executables = filter(is_executable, directory.iterdir())
        candidates = ', '.join(x.name for x in sorted(executables))
        form = 'Missing entrypoint %r! Use one of these: %s, or create it!'
        raise FileNotFoundError(form % (entrypoint, candidates))

    def _setup(self, directory, package, variables, context):
        'Highlevel method of setup steps, details are in called methods.'
        self._setup_workarea(directory)
        self._unwrap(package, directory)
        environment = os.environ.copy()
        self._prepend_paths(environment, PYTHONPATH, PYTHONPATH_EXTENSIONS)
        self._prepend_paths(environment, PATH, EXECUTABLES)
        if variables:
            self._inject_variables(environment, variables)
        self._setup_authorization(context, environment)
        entrypoints = find_all_entrypoints(directory)
        return entrypoints, environment

    def _inject_variables(self, environment, variables):
        'To get variables JSON file injected into run environment'
        with open(variables) as source:
            for key, value in json.load(source).items():
                environment[key] = stringify(value)

    def _setup_authorization(self, context, environment):
        'PLAN: Placeholder for authorization step, not done yet.'
        workspace = context.get('workspace', None)
        if not workspace:
            return False
        account = context.get('account', None)
        config = context.get('config', None)
        setup = dict(account=account, config=config, minutes=1000,
                     workspace=workspace, silent=True)
        authorize = Authorize(setup)
        _, result = main.decorated_run(authorize)
        reply = result.get('@authorization', None)
        self._inject_authorization(reply, environment)
        return True

    def _inject_authorization(self, reply, environment):
        endpoint = reply.get('endpoint', None)
        for name in ENDPOINT_VARIABLES:
            environment[name] = endpoint or NOT_AVAILABLE
        token = reply.get('token', None)
        for name in TOKEN_VARIABLES:
            environment[name] = token or NOT_AVAILABLE

    def _unwrap(self, package, directory):
        '''
        To setup and call unwrap command and extract package into correct
        directory.
        '''
        unwrap = Unwrap(dict(directory=directory, package=package))
        return unwrap(dict())

    def _prepend_paths(self, target, name, parts):
        'To prepend given paths before current values.'
        previous = target.get(name, None)
        target[name] = make_path(*parts, previous)
        return target

    def _setup_workarea(self, directory):
        'To cleanup previous content, and make sure directory exists.'
        if directory.exists():
            cleanup(directory)
        directory.mkdir(parents=True, exist_ok=True)
        return True

    def _execute(self, root, environment, entrypoint):
        '''
        To actually execute given entrypoint, with given root as working
        directory, with given environment as environment. Then wait undefined
        amount of time for task to complete. This is the end game of run
        command.
        '''
        task = subprocess.Popen((entrypoint,), cwd=root, env=environment)
        return task.wait()
