'''
This module provides supporting functions for commands. Helpers.
'''
import json
import os
import shutil


ENTRYPOINTS = 'entrypoints'
BIN = 'bin'
EXECUTABLES = (ENTRYPOINTS, BIN)

NON_EXECUTABLES = ('.md', '.txt')


def make_path(*parts):
    "Convert parts into OS path separated entries."
    valid = tuple(filter(bool, parts))
    return os.pathsep.join(valid) if valid else None


def cleanup(directory):
    "Cleaning up given directory using shutil.rmtree"
    shutil.rmtree(directory, ignore_errors=True)
    return True


def find_entrypoints(directory):
    'To find directory of entrypoints, (deeply) inside unwrapped package.'
    subdirs = list()
    for entry in directory.iterdir():
        if entry.is_dir() and entry.name == ENTRYPOINTS:
            return entry
        if entry.is_dir():
            subdirs.append(entry)
    for subdir in subdirs:
        found = find_entrypoints(subdir)
        if found:
            return found
    return None


def _find_all_entrypoints(directory, result):
    subdirs = list()
    for entry in directory.iterdir():
        if entry.is_dir() and entry.name == ENTRYPOINTS:
            result.append(entry)
        if entry.is_dir():
            subdirs.append(entry)
    for subdir in subdirs:
        _find_all_entrypoints(subdir, result)
    return result


def find_all_entrypoints(directory):
    'To find all directories of entrypoints, inside unwrapped package.'
    return tuple(_find_all_entrypoints(directory, list()))


def is_executable(filename):
    'Predicate to filter out only possible executable files.'
    is_dir = filename.is_dir()
    bad_suffix = filename.suffix.lower() in NON_EXECUTABLES
    return not (is_dir or bad_suffix)


def make_executable(directory, verbose, dryrun):
    '''
    To unconditionally make all files inside given directory as executable.
    '''
    silent = not (verbose or dryrun)
    for entry in filter(is_executable, directory.iterdir()):
        if os.access(entry, os.X_OK):
            continue
        if not silent:
            print('    - making %s executable' % entry)
        if not dryrun:
            entry.chmod(0o755)
    return True


def prepare_executables(root, verbose=False, dryrun=False):
    '''
    To make sure that files in executables directory are actually executable.
    '''
    silent = not (verbose or dryrun)
    for directory in EXECUTABLES:
        target = root.joinpath(directory)
        if target.is_dir():
            if not silent:
                print('  + processing directory: %s' % target)
            make_executable(target, verbose, dryrun)
    return True


def stringify(value):
    '''
    This function converts simple values to strings using str() form.
    But lists, tuples, and dictionaries are converted to JSON form.
    '''
    if isinstance(value, (dict, list, tuple)):
        return json.dumps(value)
    return str(value)
