'''
This module provides unwrap command to convert rpa package back to
directory structure.
'''

import zipfile
import pathlib
from .abstract_ import CommandBase, everyparser
from .fix import quickfix


def subparser(parser, argbloks):
    'To provide correct parser arguments needed by unwrap command.'
    argbloks.optional_force_flag(parser)
    argbloks.default_directory_argument(parser)
    argbloks.default_package_argument(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Unwrap command unpacks a package back to directory structure.

    It expects to get package filename, and target directory.
    And using --force option, files will be overwritten.
    '''
    def __call__(self, context):
        'Actual Command entry point when actually executing.'
        local_context = self._full_context(context)
        force = local_context.get('force', False)
        zipname = local_context.get('package', None)
        directory = pathlib.Path(local_context.get('directory', '.'))
        self._validate(zipname, directory)
        if not force:
            self._validate_no_overwrite(zipname, directory)
        self._unzip(zipname, directory)
        return quickfix(directory)

    def _validate(self, zipname, directory):
        '''
        Given arguments must be correct. If they are not, then exception
        is thrown, that should give information, what actually went wrong.
        '''
        if not directory.is_dir():
            raise NotADirectoryError('Invalid directory: %s' % directory)
        if not pathlib.Path(zipname).is_file():
            raise FileNotFoundError('Missing package file: %s' % zipname)
        return True

    def _toplevel_content(self, zipname):
        '''
        Helper function for finding out what top level files and directories
        are inside a zipfile.
        '''
        result = set()
        with zipfile.ZipFile(zipname) as archive:
            for item in archive.infolist():
                fullname = pathlib.Path(item.filename)
                result.add(fullname.parts[0])
        return tuple(sorted(result))

    def _validate_no_overwrite(self, zipname, directory):
        '''
        To provide safety for new users, this function validates, that
        unwrapping does not overwrite anything in users system.
        '''
        basedir = pathlib.Path(directory)
        for entry in self._toplevel_content(zipname):
            candidate = basedir.joinpath(entry)
            if candidate.exists():
                raise FileExistsError('Entry already exists: %s' % candidate)
        return True

    def _unzip(self, zipname, directory):
        '''
        To actually unwrap the package, zipfile extractall is used to do
        the heavy lifting.
        '''
        with zipfile.ZipFile(zipname) as archive:
            archive.extractall(path=directory)
        return True
