'''
This module provides wrap/package/zip command to convert a directory into
a rpa package, that can be uploaded to cloud.
'''

import pathlib
import zipfile
from .abstract_ import CommandBase, everyparser


DEFAULT_IGNORES = ('.git', '.rpa', 'robocode', 'tmp', 'temp', '__pycache__')


def subparser(parser, argbloks):
    '''
    To provide correct parser arguments needed by wrap command.
    '''
    argbloks.optional_force_flag(parser)
    argbloks.default_directory_argument(parser)
    argbloks.default_package_argument(parser)
    argbloks.default_ignore_argument(parser)
    return everyparser(parser, argbloks)


class Command(CommandBase):
    '''
    Wrap command builds a package out of directory content.

    It expects to get package filename, source directory and optional
    ignore file.

    When --force flag is given, existing package file will be overwritten.
    '''
    def __call__(self, context):
        '''
        Actual executed command. This method is actual package wrapping.
        Given context will be mixed with creation context, where this call
        context overwrites creation context where common keys exists.
        '''
        local_context = self._full_context(context)
        force = local_context.get('force', False)
        filename = local_context.get('package', None)
        ignore = local_context.get('ignore', None)
        directory = pathlib.Path(local_context.get('directory', '.'))

        self._validate(force, filename, directory, ignore)
        patterns = (filename,) + self._load_ignore_patterns(ignore)

        with zipfile.ZipFile(filename,
                             mode='w',
                             compression=zipfile.ZIP_DEFLATED,
                             compresslevel=9) as sink:
            context = directory.absolute()
            self._add_content(context.parent, context, patterns, sink)
        if self.debug:
            self._list_content(filename)
        return True

    def _validate(self, force, filename, directory, ignore):
        '''
        Given arguments must be correct. If they are not, then exception
        is thrown, that should give information, what actually went wrong.
        '''
        filepath = pathlib.Path(filename)
        if not directory.is_dir():
            raise NotADirectoryError('Invalid directory: %s' % directory)
        if not filepath.parent.is_dir():
            raise NotADirectoryError('Invalid directory: %s' % filepath.parent)
        if ignore and not pathlib.Path(ignore).is_file():
            raise FileNotFoundError('Missing ignore file: %s' % ignore)
        if not force and filepath.exists():
            form = 'Package already exists: %s (use --force to overwrite it).'
            raise FileExistsError(form % filename)
        return True

    def _load_ignore_patterns(self, filename):
        '''
        To be able to have content that does not end up in package.zip,
        this method loads .gitignore like file content as effective ignores.
        '''
        if not filename:
            return DEFAULT_IGNORES
        path = pathlib.Path(filename)
        result = []
        with open(path) as source:
            for line in source:
                stripped_line = line.strip()
                if stripped_line:
                    result.append(stripped_line)
        return DEFAULT_IGNORES + tuple(result)

    def _list_content(self, filename):
        '''
        For debugging purposes, it is good to see listing of zip content,
        after the package has been created.
        '''
        print('Content of %s file:' % filename)
        with zipfile.ZipFile(filename, 'r') as source:
            for item in source.namelist():
                print('- %s' % item)

    def _add_content(self, base, directory, patterns, package):
        '''
        To be able to have tree-structure like content in zip, this
        recursive function travels given directory and adds content into
        package, and using patterns as a way to ignore unwanted entries.
        '''
        for entry in directory.iterdir():
            if any(entry.match(x) for x in patterns):
                continue
            if entry.is_dir():
                self._add_content(base, entry, patterns, package)
            else:
                package.write(entry, entry.relative_to(base))
