'''
This module provides Config and Account objects, that are used to handle
robocode robocode.yaml configuration files.
'''
import pathlib
import time
import yaml

UNIQUE_MARKER = object()


def nicepath(filename):
    '''
    Helper function to expand user directory and convert given path to
    absolute path representation.
    '''
    path = pathlib.Path(filename.strip())
    return path.expanduser().absolute()


class _Core:
    FIELDS = tuple()

    def update(self, details):
        '''
        Update method is used to import dictionary values into account
        object.
        '''
        for key in self.FIELDS:
            setattr(self, key, details.get(key, None))
        return self

    @property
    def as_dict(self):
        '''
        This method converts internals of Account object into dictionary
        entry, that can be serialize using yaml.
        '''
        result = dict()
        for key in self.FIELDS:
            result[key] = getattr(self, key, None)
        return result

    def __lt__(self, other):
        for field in self.FIELDS:
            mine = getattr(self, field, None)
            they = getattr(other, field, None)
            if mine == they:
                continue
            if mine is None and they is not None:
                return True
            if they is None:
                return False
            if mine < they:
                return True
        return id(self) < id(other)


class Activity(_Core):
    '''
    Activity class to support activity handling.
    '''
    FIELDS = ('updated', 'created', 'activity', 'deleted')

    def __init__(self, **kvargs):
        '''
        Activity can be initialized with updated, created, activity, and
        deleted values.
        '''
        self.activity = self.created = self.updated = self.deleted = None
        for key in self.FIELDS:
            setattr(self, key, kvargs.get(key, None))


class Account(_Core):
    '''
    Account captures robocloud account information as python object.
    '''
    FIELDS = ('account', 'endpoint', 'identifier', 'secret')

    def __init__(self, **kvargs):
        '''
        Account can be initialized with account name, endpoint, identifier,
        and secret values.
        '''
        self.account = self.endpoint = None
        self.identifier = self.secret = None
        for key in self.FIELDS:
            setattr(self, key, kvargs.get(key, None))


class Config:
    '''
    Config represents configuration needed for robocode/robolab to access
    robocloud resources. Thru configuration you can get Account object if
    one is found.
    '''
    def __init__(self):
        '''
        Initializes internal state of Config object, like empty list of
        account informations.
        '''
        self._content = list()

    def __len__(self):
        '''
        Method to see, how many accounts there is currently defined.
        '''
        return len(self._content)

    def load_from(self, filename):
        '''
        Loads accounts from given YAML file, and stores it into internal
        instance variable.
        '''
        fullpath = nicepath(filename)
        if not fullpath.is_file():
            return False
        with open(fullpath) as source:
            found = tuple(yaml.safe_load_all(source))
            self._content = list(found[0]) if len(found) == 1 else list()
        return True

    def save_as(self, filename):
        '''
        Saves current content into given filename, as YAML document.
        '''
        with open(nicepath(filename), 'w+') as sink:
            yaml.safe_dump(self._content, sink)
        return True

    def find_account(self, account):
        '''
        Way to find account from list of accounts, by account name.
        If account is not found, then default account is returned.
        If default account is not defined, returns None.
        '''
        selector = self._index_of(account)
        if selector is None:
            selector = self._index_of('default')
        if selector is None:
            return None
        return Account().update(self._content[selector])

    def update_account(self, account):
        '''
        This method is used to update or add accounts into this config
        instance.
        '''
        selector = self._index_of(account.account)
        if selector is None:
            self._content.append(account.as_dict)
        else:
            self._content[selector].update(account.as_dict)

    def _index_of(self, name, kind='account'):
        '''
        Helper method for finding index of named account from internal
        list of account (dictionaries).
        '''
        for index, it in enumerate(self._content):
            if name == it.get(kind, UNIQUE_MARKER):
                return index
        return None

    def summon_activity(self, path):
        '''
        Method for summoning activity (creating or updating) with touching
        it as an update.
        '''
        where = pathlib.Path(path).absolute()
        selector = self._index_of(str(where), 'activity')
        now = time.time()
        if selector is None:
            activity = Activity(activity=str(where), created=now, updated=now)
            self._content.append(activity.as_dict)
        else:
            activity = Activity().update(self._content[selector])
            activity.updated = now
            activity.deleted = None
            self._content[selector].update(activity.as_dict)
        return activity

    def delete_activity(self, path):
        '''
        Method for marking activity deleted if it exists.
        '''
        now = time.time()
        fullpath = pathlib.Path(path).absolute()
        selector = self._index_of(str(fullpath), 'activity')
        if selector is not None:
            activity = Activity().update(self._content[selector])
            activity.deleted = now
            self._content[selector].update(activity.as_dict)
        return selector is not None

    def _filter(self, kind):
        for it in self._content:
            if it.get(kind, None):
                yield it

    @property
    def activities(self):
        '''
        Loop thru all activities known in history context.
        '''
        for it in self._filter('activity'):
            yield Activity().update(it)

    @property
    def accounts(self):
        '''List all available accounts'''
        for it in self._filter('account'):
            yield Account().update(it)
