#!/usr/bin/python3

'Main module for robocode CLI implementation.'

import argparse
import glob
import importlib
import sys
import traceback
from os import path
from collections import namedtuple
from .authorization import AuthorizationDecorator
from . import argbloks

Entry = namedtuple('Entry', 'command subparser')

BASEDIR = path.realpath(path.dirname(__file__))
COMMANDDIR = path.join(BASEDIR, 'commands')
COMMAND_MODULE = 'robocode.commands'

ORIGINAL_ARGS = tuple(sys.argv[1:])


def load_command(name, module=COMMAND_MODULE):
    'This method is used to load parser and command from package.'
    try:
        found = importlib.import_module('%s.%s' % (module, name))
        return Entry(found.Command, found.subparser)
    except:  # pylint: disable=bare-except
        return None


def find_commands(directory=COMMANDDIR, module=COMMAND_MODULE):
    'This method is used to find all commands from command directory.'
    result = dict()
    for filename in glob.glob(path.join(directory, '*.py')):
        basename, _ = path.splitext(path.basename(filename))
        if basename.endswith('_'):
            continue
        found = load_command(basename, module=module)
        if found:
            result[basename.lower()] = found
    return result


def decorated_run(command):
    'Runs given command with decorators and returns status and result context.'
    task = AuthorizationDecorator(command)
    result = dict()
    success = task(result)
    return success, result


def process(options):
    'This method is to be central place for actual action performed.'
    task = load_command(options.command)
    core = task.command(dict(options.__dict__))
    success, _ = decorated_run(core)
    return 0 if success else 4


def commandline(args=ORIGINAL_ARGS, directory=COMMANDDIR,
                module=COMMAND_MODULE):
    '''
    Robocode CLI -- command line tool for robocloud.

    Version number:
    '''
    documentation = commandline.__doc__.strip() + ' ' + argbloks.read_version()
    parser = argparse.ArgumentParser(description=documentation)
    argbloks.optional_version_flag(parser)
    subparsers = parser.add_subparsers(
        help='command to be executed',
        dest='command')
    entries = tuple(sorted(find_commands(directory, module).items()))
    for name, (command, subparser) in entries:
        if name.startswith('_'):
            continue
        full = command.__doc__.strip()
        description = full.splitlines().pop(0)
        subparser(subparsers.add_parser(name,
                                        description=full,
                                        help=description), argbloks)
    result = parser.parse_args(args)
    if not result.command:
        parser.print_help()
        sys.exit(-1)
    return result


def main(args=ORIGINAL_ARGS):
    'This wrapper method can be exposed as command to operating system.'
    debug = False
    try:
        options = commandline(args)
        debug = getattr(options, 'debug', False)
        return process(options)
    except Exception as failure:  # pylint: disable=broad-except
        if debug:
            traceback.print_exc()
            print()
        print('Failure: %s' % failure)
        return 4

if __name__ == '__main__':
    sys.exit(main())
